@@ -12,17 +12,23 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from .. import mlog
+from __future__ import annotations
+import typing as T
+
 from .qt import QtBaseModule
+from . import ModuleInfo
+
+if T.TYPE_CHECKING:
+    from ..interpreter import Interpreter
 
 
 class Qt5Module(QtBaseModule):
 
-    def __init__(self, interpreter):
+    INFO = ModuleInfo('qt5')
+
+    def __init__(self, interpreter: Interpreter):
         QtBaseModule.__init__(self, interpreter, qt_version=5)
 
 
-def initialize(*args, **kwargs):
-    mlog.warning('rcc dependencies will not work reliably until this upstream issue is fixed:',
-                 mlog.bold('https://bugreports.qt.io/browse/QTBUG-45460'), fatal=False)
-    return Qt5Module(*args, **kwargs)
+def initialize(interp: Interpreter) -> Qt5Module:
+    return Qt5Module(interp)
