@@ -119,7 +119,7 @@ class Parser:
         return Statement(cur.value, args)
 
     def arguments(self) -> T.List[T.Union[Token, T.Any]]:
-        args = []  # type: T.List[T.Union[Token, T.Any]]
+        args: T.List[T.Union[Token, T.Any]] = []
         if self.accept('lparen'):
             args.append(self.arguments())
             self.expect('rparen')
@@ -159,7 +159,7 @@ class Converter:
         self.cmake_root = Path(cmake_root).expanduser()
         self.indent_unit = '  '
         self.indent_level = 0
-        self.options = []  # type: T.List[tuple]
+        self.options: T.List[T.Tuple[str, str, T.Optional[str]]] = []
 
     def convert_args(self, args: T.List[Token], as_array: bool = True) -> str:
         res = []
@@ -280,13 +280,13 @@ class Converter:
             subdir = self.cmake_root
         cfile = Path(subdir).expanduser() / 'CMakeLists.txt'
         try:
-            with cfile.open() as f:
+            with cfile.open(encoding='utf-8') as f:
                 cmakecode = f.read()
         except FileNotFoundError:
             print('\nWarning: No CMakeLists.txt in', subdir, '\n', file=sys.stderr)
             return
         p = Parser(cmakecode)
-        with (subdir / 'meson.build').open('w') as outfile:
+        with (subdir / 'meson.build').open('w', encoding='utf-8') as outfile:
             for t in p.parse():
                 if t.name == 'add_subdirectory':
                     # print('\nRecursing to subdir',
@@ -300,7 +300,7 @@ class Converter:
 
     def write_options(self) -> None:
         filename = self.cmake_root / 'meson_options.txt'
-        with filename.open('w') as optfile:
+        with filename.open('w', encoding='utf-8') as optfile:
             for o in self.options:
                 (optname, description, default) = o
                 if default is None:
